@@ -48,7 +48,12 @@ void save_mod_file(MainView *mainview);
 char *get_confirmed_pass(MainView *mainview);
 
 /* activate grid */
-void callback_activate_treeView(GtkTreeView *treeView, GtkTreePath *path, MainView *mainview) {
+void callback_activate_treeView(GtkTreeView *treeView, GtkTreePath *path, GtkTreeViewColumn *column, MainView *mainview) {
+	callback_hildon_activate_treeView(treeView, path, mainview);
+}
+
+/* activate hildon grid */
+void callback_hildon_activate_treeView(GtkTreeView *treeView, GtkTreePath *path, MainView *mainview) {
 	GtkTreeIter iter;
 	rec *onrec=NULL;
 	g_assert(mainview != NULL && mainview->data != NULL );
@@ -65,6 +70,8 @@ void callback_activate_treeView(GtkTreeView *treeView, GtkTreePath *path, MainVi
 		show_rec(mainview, onrec);
 	} else if (gtk_tree_view_collapse_row(treeView, path)) {
 		gtk_tree_view_expand_row(treeView, path, FALSE);
+	} else {
+		gtk_tree_view_collapse_row(treeView, path);
 	}
 }
 
@@ -114,7 +121,7 @@ void callback_rec_response(GtkDialog *dialog, gint arg1, MainView *mainview) {
 	switch (arg1) {
 		case 1: /* save any changes */
 			safeReplaceString(&rec->title, gtk_entry_get_text(GTK_ENTRY(recwindow->title)), mainview);
-			safeReplaceString(&rec->group, gtk_entry_get_text(GTK_ENTRY(recwindow->group)), mainview);
+			safeReplaceString(&rec->group, gtk_combo_box_get_active_text(GTK_COMBO_BOX(recwindow->group)), mainview);
 			safeReplaceString(&rec->user, gtk_entry_get_text(GTK_ENTRY(recwindow->username)), mainview);
 			safeReplaceString(&rec->password, gtk_entry_get_text(GTK_ENTRY(recwindow->password)), mainview);
 			buffer=gtk_text_view_get_buffer (GTK_TEXT_VIEW(recwindow->notes));
@@ -144,6 +151,7 @@ void callback_rec_response(GtkDialog *dialog, gint arg1, MainView *mainview) {
 			interface_set_rec_editable(recwindow, TRUE);
 			break;
 		case -4:
+			//Cancel fro Fremantle (out of dialog click)
 			unshow_rec(mainview);  /* close rec window */
 			populate_treeView(mainview); /* redraw grid */
 			break;
