@@ -27,6 +27,7 @@
 #include <src/preferences.h>
 #include <gtk/gtk.h>
 #include <libintl.h>
+#include <stdlib.h>
 #include <string.h>
 
 #ifdef HAVE_CONFIG_H
@@ -54,7 +55,7 @@ static void create_toolbar( MainView *main );
 void create_textarea( MainView *main );
 GtkWidget *show_rec_line(GtkWidget *sizer, char *label, char *txt, int row, int col);
 
-inline char *emptynull(char *a) { return (a==NULL) ? "" : a;}
+char *emptynull(char *a) { return (a==NULL) ? "" : a;}
 
 /* Creates and initialises a main_view */
 MainView* interface_main_view_new( AppData *data ) {
@@ -64,7 +65,7 @@ MainView* interface_main_view_new( AppData *data ) {
 /*
 	GtkWidget *main_vbox = gtk_vbox_new( FALSE, 0 );
 */
-#if MAEMO_VERSION_MAJOR == 5
+#if MAEMO_VERSION_MAJOR >= 5
 	scrolled_window = hildon_pannable_area_new();
 #else
 	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
@@ -135,6 +136,17 @@ gchar *get_password(MainView *main, gchar *caption) {
 	return result;
 }
 
+gboolean fillCombo (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GtkWidget *combo) {
+	rec *data;
+	gchar *label;
+	
+	gtk_tree_model_get(model, iter, LABEL_COLUMN, &label, DATA_COLUMN, &data,  -1);
+	if (data == NULL && label != NULL) {
+		gtk_combo_box_append_text(GTK_COMBO_BOX(combo), label);
+	}
+	return FALSE;
+}
+
 void show_rec(MainView *main, rec *r) {
 	GtkWidget *sizer, *scrollwindow;
 	GtkTextBuffer *buffer=NULL;
@@ -143,18 +155,28 @@ void show_rec(MainView *main, rec *r) {
 	rec_win=g_malloc0(sizeof(RecWindow));
 	main->rec_win=rec_win;
 	rec_win->rec=r;
-	rec_win->dialog = gtk_dialog_new_with_buttons ("Edit/View Entry", GTK_WINDOW(main->data->main_view), GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR, GTK_STOCK_OK, 1, "Copy Username", 2, "Copy Password", 3, NULL);
+	rec_win->dialog = gtk_dialog_new_with_buttons (_("Edit/View Entry"), GTK_WINDOW(main->data->main_view), GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR, GTK_STOCK_OK, 1, _("Copy User"), 2, _("Copy Pwd"), 3, NULL);
 	rec_win->delete_button = gtk_dialog_add_button(GTK_DIALOG(rec_win->dialog), GTK_STOCK_DELETE, 4);
-	rec_win->edit_button = gtk_dialog_add_button(GTK_DIALOG(rec_win->dialog), "Edit", 5);
+	rec_win->edit_button = gtk_dialog_add_button(GTK_DIALOG(rec_win->dialog), _("Edit"), 5);
 /*	gtk_button_set_image(GTK_BUTTON(rec_win->edit_button), GTK_STOCK_EDIT); */
-    g_object_set(rec_win->dialog, "content-area-border", 1, "button_spacing", 20, NULL);
+	g_object_set(rec_win->dialog, "content-area-border", 1, "button_spacing", 20, NULL);
     /*  where's "button-spacing", "content-area-border", "action_area_border" ? - not in scratchbox, maybe on n800 */
 	g_signal_connect(rec_win->dialog, "response", G_CALLBACK ( callback_rec_response), main);
 	sizer=gtk_table_new(5, 4, TRUE);
-	rec_win->title=show_rec_line(sizer, "Title", r->title, 0, 0);
-	rec_win->group=show_rec_line(sizer, "Group", r->group, 0, 2);
-	rec_win->username=show_rec_line(sizer, "Username", r->user, 1, 0);
-	rec_win->password=show_rec_line(sizer, "Password", r->password, 1, 2);
+	rec_win->title=show_rec_line(sizer, _("Title"), r->title, 0, 0);
+//	rec_win->group=show_rec_line(sizer, "Group", r->group, 0, 2);
+	// Change group entry to ComboBox with Entry
+	gtk_table_attach_defaults(GTK_TABLE(sizer), gtk_label_new(_("Group")), 2, 3, 0, 1);
+	rec_win->group=gtk_combo_box_entry_new_text();
+	if (r->group!=NULL) {
+		gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(rec_win->group))), r->group);
+	}
+	gtk_table_attach_defaults(GTK_TABLE(sizer), rec_win->group, 3, 4, 0, 1);
+	// Fill the combo
+	gtk_tree_model_foreach(GTK_TREE_MODEL(main->store), (GtkTreeModelForeachFunc) fillCombo, (gpointer) rec_win->group);
+	
+	rec_win->username=show_rec_line(sizer, _("Username"), r->user, 1, 0);
+	rec_win->password=show_rec_line(sizer, _("Password"), r->password, 1, 2);
 
 	rec_win->notes=gtk_text_view_new();
 	buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(rec_win->notes));
@@ -240,13 +262,23 @@ void populate_treeView (MainView *main) {
 gchar* interface_file_chooser(MainView * mainview, GtkFileChooserAction action) {
 	GtkWidget *dialog;
 	gchar* filename = NULL;
-
+	const gchar *docsDir = NULL;
+	gchar *myDir = NULL;
+	
+	docsDir = getenv("MYDOCSDIR");
+	if (docsDir != NULL) {
+		myDir = g_strconcat(docsDir, "/.documents", NULL);
+	} else {
+		myDir = g_strdup("/home/user/MyDocs/.documents");
+	}
 	dialog = hildon_file_chooser_dialog_new( GTK_WINDOW(mainview->data->main_view), action);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), myDir);
 	gtk_widget_show_all (GTK_WIDGET(dialog));
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
 		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
 	}
 	gtk_widget_destroy (dialog);
+	g_free(myDir);
 	return filename;
 }
 
@@ -301,20 +333,34 @@ void interface_error( PwsafeError me, MainView *main )
     g_free( msg );
 }
 
+#if MAEMO_VERSION_MAJOR >= 5
+static GtkWidget* add_menu_item (GtkWidget *main_menu, const gchar* title,
+		    GCallback clicked_callback, gpointer user_data) {
+    HildonAppMenu *app_menu = HILDON_APP_MENU (main_menu);
+    /* Create a button, add it, and return it: */ 
+    GtkWidget *button = hildon_button_new_with_text (HILDON_SIZE_AUTO, 
+	            HILDON_BUTTON_ARRANGEMENT_VERTICAL, title, NULL);
+    gtk_widget_show (button);
+    g_signal_connect_after (button, "clicked", G_CALLBACK (clicked_callback), user_data);
+    hildon_app_menu_append (app_menu, GTK_BUTTON (button));
+    return button;
+}
+#endif
 
 /* Create the menu items needed for the drop down menu */
-static void create_menu( MainView *main )
-{
+static void create_menu( MainView *main ) {
 	/* Create needed handles */
-	GtkMenu *main_menu;
+	GtkWidget *main_menu;
+
+#if MAEMO_VERSION_MAJOR < 5
 	GtkWidget *file_menu, *view_menu;
 	GtkWidget *separator = NULL;
 	GtkWidget *quit = NULL;
 
 	/* Get the menu of our view */
-	main_menu = GTK_MENU( gtk_menu_new () );
+	main_menu = gtk_menu_new ();
 	/* Add menu to HildonWindow */
-	hildon_window_set_menu(main->data->main_view, main_menu); 
+	hildon_window_set_menu(main->data->main_view, GTK_MENU(main_menu)); 
 	/* Create new menus for submenus in our drop down menu */
 	file_menu = gtk_menu_new ();
 	view_menu = gtk_menu_new ();
@@ -327,7 +373,7 @@ static void create_menu( MainView *main )
 	main->close_item = gtk_menu_item_new_with_label ( _("Close") );
 	main->save_item = gtk_menu_item_new_with_label ( _("Save") );
 	main->saveas_item = gtk_menu_item_new_with_label ( _("Save As...") );
-    main->export_item = gtk_menu_item_new_with_label ( _("Export...") );
+	main->export_item = gtk_menu_item_new_with_label ( _("Export...") );
 	
 	main->fullscreen_item = gtk_menu_item_new_with_label ( _("Full Screen") );
 	main->add_item = gtk_menu_item_new_with_label ( _("Add Entry") );
@@ -336,20 +382,20 @@ static void create_menu( MainView *main )
 	quit = gtk_menu_item_new_with_label( _("Quit") );
 
 	/* Add menu items to right menus */
-	gtk_menu_append( main_menu, main->file_item );
-	gtk_menu_append( file_menu, main->new_item );
-	gtk_menu_append( file_menu, main->open_item );
-	gtk_menu_append( file_menu, main->open_last_item );
-	gtk_menu_append( file_menu, main->close_item );
-	gtk_menu_append( file_menu, main->save_item );
-	gtk_menu_append( file_menu, main->saveas_item );
-    gtk_menu_append( file_menu, main->export_item );
+	gtk_menu_append( GTK_MENU(main_menu), main->file_item );
+	gtk_menu_append( GTK_MENU(file_menu), main->new_item );
+	gtk_menu_append( GTK_MENU(file_menu), main->open_item );
+	gtk_menu_append( GTK_MENU(file_menu), main->open_last_item );
+	gtk_menu_append( GTK_MENU(file_menu), main->close_item );
+	gtk_menu_append( GTK_MENU(file_menu), main->save_item );
+	gtk_menu_append( GTK_MENU(file_menu), main->saveas_item );
+	gtk_menu_append( GTK_MENU(file_menu), main->export_item );
  
-	gtk_menu_append( main_menu, main->add_item );
-	gtk_menu_append( main_menu, main->change_password );
-	gtk_menu_append( main_menu, main->fullscreen_item );
-	gtk_menu_append( main_menu, separator );
-	gtk_menu_append( main_menu, quit );
+	gtk_menu_append( GTK_MENU(main_menu), main->add_item );
+	gtk_menu_append( GTK_MENU(main_menu), main->change_password );
+	gtk_menu_append( GTK_MENU(main_menu), main->fullscreen_item );
+	gtk_menu_append( GTK_MENU(main_menu), separator );
+	gtk_menu_append( GTK_MENU(main_menu), quit );
 
 	/* Add submenus to the right items */
 	gtk_menu_item_set_submenu( GTK_MENU_ITEM (main->file_item), file_menu );
@@ -362,30 +408,62 @@ static void create_menu( MainView *main )
 	g_signal_connect( G_OBJECT( main->close_item ), "activate",  G_CALLBACK ( callback_file_close), main );
 	g_signal_connect( G_OBJECT( main->save_item ), "activate",  G_CALLBACK ( callback_file_save), main );
 	g_signal_connect( G_OBJECT( main->saveas_item ), "activate",  G_CALLBACK ( callback_file_saveas), main );
-    g_signal_connect( G_OBJECT( main->export_item ), "activate",  G_CALLBACK ( callback_export), main );
+	g_signal_connect( G_OBJECT( main->export_item ), "activate",  G_CALLBACK ( callback_export), main );
 	g_signal_connect( G_OBJECT( main->add_item ), "activate",  G_CALLBACK ( callback_add_entry), main );
 	g_signal_connect( G_OBJECT( main->change_password ), "activate",  G_CALLBACK ( callback_change_password), main );
 	g_signal_connect( G_OBJECT( main->fullscreen_item ), "activate",  G_CALLBACK ( callback_fullscreen ), main );
 	g_signal_connect( G_OBJECT( quit ), "activate", gtk_main_quit, NULL );
+#else
+/* Maemo 5+ */
+	/* Get the menu of our view */
+	main_menu = hildon_app_menu_new ();
+
+	/* Create the menu items */
+	main->new_item = add_menu_item (main_menu, _("New"), G_CALLBACK ( callback_file_new), main );
+	main->open_item = add_menu_item (main_menu, _("Open..."), G_CALLBACK ( callback_file_open), main );
+	main->open_last_item = add_menu_item (main_menu, _("Open Last"), G_CALLBACK ( callback_file_open_last), main );
+	main->close_item = add_menu_item (main_menu, _("Close"), G_CALLBACK ( callback_file_close), main );
+	main->save_item = add_menu_item (main_menu, _("Save"), G_CALLBACK ( callback_file_save), main );
+	main->saveas_item = add_menu_item (main_menu, _("Save As..."), G_CALLBACK ( callback_file_saveas), main );
+	main->export_item = add_menu_item (main_menu, _("Export..."), G_CALLBACK ( callback_export), main );
+	
+	main->change_password = add_menu_item (main_menu, _("Change Password"), G_CALLBACK ( callback_change_password), main );
+
+	/* Add menu to HildonWindow */
+	hildon_window_set_app_menu (HILDON_WINDOW (main->data->main_view), HILDON_APP_MENU (main_menu));
+#endif
 
 	/* We need to show menu items */
 	gtk_widget_show_all( GTK_WIDGET( main_menu ) );
 }
 
+void interface_show_item(GtkWidget *menuWidget, gboolean show) {
+	if (GTK_IS_WIDGET(menuWidget)) { 
+#if MAEMO_VERSION_MAJOR < 5
+		gtk_widget_set_sensitive(menuWidget, show);
+#else
+		if (show) {
+			gtk_widget_show(menuWidget);
+		} else {
+			gtk_widget_hide(menuWidget);
+		}
+#endif
+	}
+}
+
 void interface_show_state(MainView *main) {
 	gboolean fopen;
 	fopen=(main->pass!=NULL);
-    if (GTK_IS_WIDGET(main->new_item)) {    /* ignore menu if it's not there (eg. during app close) */
-    	gtk_widget_set_sensitive(main->new_item, !fopen);
-	    gtk_widget_set_sensitive(main->open_item, !fopen);
-    	gtk_widget_set_sensitive(main->open_last_item, main->last_file && (!fopen));
-    	gtk_widget_set_sensitive(main->close_item, fopen);
-    	gtk_widget_set_sensitive(main->save_item, fopen);
-    	gtk_widget_set_sensitive(main->saveas_item, fopen);
-        gtk_widget_set_sensitive(main->export_item, fopen);
-    	gtk_widget_set_sensitive(main->change_password, fopen);
-    	gtk_widget_set_sensitive(main->add_item, fopen);
-    }
+
+    	interface_show_item(main->new_item, !fopen);
+	interface_show_item(main->open_item, !fopen);
+    	interface_show_item(main->open_last_item, main->last_file && (!fopen));
+    	interface_show_item(main->close_item, fopen);
+    	interface_show_item(main->save_item, fopen);
+    	interface_show_item(main->saveas_item, fopen);
+        interface_show_item(main->export_item, fopen);
+    	interface_show_item(main->change_password, fopen);
+	interface_show_item(main->add_item, fopen);
 	
 	if (GTK_IS_WIDGET(main->new_tb)) {	/* ignore toolbar if it's not there (eg. during app close) */
 		gtk_widget_set_sensitive(GTK_WIDGET(main->new_tb), !fopen);
@@ -395,7 +473,7 @@ void interface_show_state(MainView *main) {
 		gtk_widget_set_sensitive(GTK_WIDGET(main->saveas_tb), fopen);
 		gtk_widget_set_sensitive(GTK_WIDGET(main->change_password_tb), fopen);
 		gtk_widget_set_sensitive(GTK_WIDGET(main->add_tb), fopen);
-        gtk_widget_set_sensitive(GTK_WIDGET(main->filter_tb), fopen);
+		gtk_widget_set_sensitive(GTK_WIDGET(main->filter_tb), fopen);
 	}
 }
 
@@ -409,7 +487,8 @@ void interface_set_rec_editable(RecWindow* rec_win, gboolean editable) {
 		gtk_widget_show(GTK_WIDGET(rec_win->edit_button));
 	}
 	gtk_editable_set_editable(GTK_EDITABLE(rec_win->title), editable);
-	gtk_editable_set_editable(GTK_EDITABLE(rec_win->group), editable);
+	// Group is a comboBoxEdit
+	gtk_editable_set_editable(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(rec_win->group))), editable);
 	gtk_editable_set_editable(GTK_EDITABLE(rec_win->username), editable);
 	gtk_editable_set_editable(GTK_EDITABLE(rec_win->password), editable);
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(rec_win->notes), editable);
@@ -441,7 +520,13 @@ static void create_toolbar ( MainView *main ) {
     /* create filter text entry in a container */
     main->filter_item_tb = gtk_tool_item_new();
     gtk_tool_item_set_expand(main->filter_item_tb, TRUE); 
+#if MAEMO_VERSION_MAJOR < 5
     main->filter_tb = gtk_entry_new();
+#else
+    main->filter_tb = hildon_entry_new(HILDON_SIZE_AUTO);
+    hildon_entry_set_placeholder (HILDON_ENTRY (main->filter_tb), _("Filter"));
+#endif
+
     /* gtk_entry_set_max_length( GTK_ENTRY(main->filter_tb) , (gint) 24);  */
     gtk_container_add( GTK_CONTAINER( main->filter_item_tb ) , main->filter_tb);
 	
@@ -455,9 +540,11 @@ static void create_toolbar ( MainView *main ) {
 	gtk_toolbar_insert ( GTK_TOOLBAR(main->toolbar), main->add_tb, -1);
 	gtk_toolbar_insert ( GTK_TOOLBAR(main->toolbar), main->change_password_tb, -1);
 	gtk_toolbar_insert ( GTK_TOOLBAR(main->toolbar), main->separator_tb2, -1);
+#if MAEMO_VERSION_MAJOR < 5
 	gtk_toolbar_insert ( GTK_TOOLBAR(main->toolbar), main->fullscreen_tb, -1);
-    gtk_toolbar_insert ( GTK_TOOLBAR(main->toolbar), main->separator_tb3, -1);
-    gtk_toolbar_insert ( GTK_TOOLBAR(main->toolbar), main->filter_item_tb, -1);
+	gtk_toolbar_insert ( GTK_TOOLBAR(main->toolbar), main->separator_tb3, -1);
+#endif
+	gtk_toolbar_insert ( GTK_TOOLBAR(main->toolbar), main->filter_item_tb, -1);
 
 	/* Connect signals to buttons */
 	g_signal_connect(G_OBJECT(main->new_tb), "clicked", G_CALLBACK(callback_file_new), main);
@@ -467,8 +554,10 @@ static void create_toolbar ( MainView *main ) {
 	g_signal_connect(G_OBJECT(main->saveas_tb), "clicked", G_CALLBACK(callback_file_saveas), main);
 	g_signal_connect(G_OBJECT(main->add_tb), "clicked", G_CALLBACK(callback_add_entry), main);
 	g_signal_connect(G_OBJECT(main->change_password_tb), "clicked", G_CALLBACK(callback_change_password), main);
+#if MAEMO_VERSION_MAJOR < 5
 	g_signal_connect(G_OBJECT(main->fullscreen_tb), "clicked", G_CALLBACK(callback_fullscreen), main);
-    g_signal_connect(G_OBJECT(main->filter_tb), "activate", G_CALLBACK(callback_filter), main);
+#endif
+	g_signal_connect(G_OBJECT(main->filter_tb), "activate", G_CALLBACK(callback_filter), main);
 	
 	/* Attach toolbar to main view */
 	hildon_window_add_toolbar(main->data->main_view, GTK_TOOLBAR(main->toolbar)); 
@@ -487,7 +576,7 @@ void create_textarea( MainView *main ) {
 	main->font_desc = pango_font_description_from_string ("Nokia Sans 16");
 	main->store=gtk_tree_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
 	main->treeView=gtk_tree_view_new_with_model(GTK_TREE_MODEL(main->store));
-#if MAEMO_VERSION_MAJOR == 5
+#if MAEMO_VERSION_MAJOR >= 5
 	gtk_tree_view_set_show_expanders(GTK_TREE_VIEW (main->treeView), TRUE);
 #endif
 	renderer = gtk_cell_renderer_pixbuf_new();
@@ -504,7 +593,11 @@ void create_textarea( MainView *main ) {
         gtk_tree_view_columns_autosize(GTK_TREE_VIEW(main->treeView));
 				
     /* Connect signals */
-	g_signal_connect (G_OBJECT(main->treeView), "hildon-row-tapped", G_CALLBACK (callback_activate_treeView), main);
+#if MAEMO_VERSION_MAJOR >= 5
+	g_signal_connect (G_OBJECT(main->treeView), "hildon-row-tapped", G_CALLBACK (callback_hildon_activate_treeView), main);
+#else
+	g_signal_connect (G_OBJECT(main->treeView), "row-activated", G_CALLBACK (callback_activate_treeView), main);
+#endif
 }
 
 /* toggle fullscreen */
