@@ -108,6 +108,9 @@ int main( int argc, char* argv[] ) {
 	/* Begin the main app */
 	gtk_widget_show ( GTK_WIDGET ( data->main_view ) );
 
+	// Manage interface
+	interface_show_state(main_view);
+		
 	main_view->last_file=conf_get_string(CONF_LASTFILE);
 	g_signal_emit_by_name(G_OBJECT(main_view->open_last_item), "activate");
 	
